--Character Randomizer V3
--Made by Nightwolf

local skinlist_str=""
local skinlist_tab={}
local crtime = CV_RegisterVar({
	name = "cr_time",
	defaultvalue = tostring(30),
	flags = CV_NETVAR,
	PossibleValue = {MIN = 1, MAX = 3600},
	func = function() end
})
local cr_ultimate = CV_RegisterVar({
	name = "cr_ultimate",
	defaultvalue = tostring(0),
	flags = CV_NETVAR,
	PossibleValue = CV_OnOff,
	func = function() end
})
local cr_sa2random = CV_RegisterVar({
	name = "cr_sa2random",
	defaultvalue = tostring(0),
	flags = CV_NETVAR,
	PossibleValue = CV_OnOff,
	func = function() end
})
local cr_timeoffset = CV_RegisterVar({
	name = "cr_timeoffset",
	defaultvalue = tostring(0),
	flags = CV_NETVAR,
	PossibleValue = {MIN = 0, MAX = 900},
	func = function() end
})
local cr_nextskin = CV_RegisterVar({
	name = "cr_shownextskin",
	defaultvalue = tostring(0),
	flags = CV_NETVAR,
	PossibleValue = CV_OnOff,
	func = function() end
})
local cr_showtimer = CV_RegisterVar({
	name = "cr_showtimer",
	defaultvalue = tostring(1),
	flags = CV_NETVAR,
	PossibleValue = CV_OnOff,
	func = function() end
})
local crandom_enable=true

//Netgame synchronization stuff (CVARs synchronize automatically, so they're not here)
addHook("NetVars", function(net)
	skinlist_str = net(skinlist_str)
	skinlist_tab = net(skinlist_tab)
	crandom_enable = net(crandom_enable)
end)

//Returns the skinlist table index if it's in the skin table or nil if it's not found
local function skintabindex(skin)
	if #skinlist_tab == 0 then return nil end
	if not skin then return nil end
	for k,v in ipairs(skinlist_tab) do
		if v == skin then return k end
	end
	return nil
end

//Checks if the skin is valid
local function isskinvalid(skin)
	for sk in skins.iterate do
		if sk.name == skin then return true end
	end
	return false
end

//Make skin list string from the skinlist table
local function makeskinstr()
	skinlist_str = ""
	for k,v in ipairs(skinlist_tab) do
		if skinlist_str == "" then skinlist_str = v else skinlist_str = $1..","..v end
	end
end

//Get a list of skins from all available skins (called by 'cr_skin reload' and after loading the mod)
local function getskinlist(p)
	skinlist_tab = {}
	for sk in skins.iterate do
		if p then CONS_Printf(p,"CharRandom: Added skin "..sk.name) end
		table.insert(skinlist_tab,sk.name)
	end
	makeskinstr()
end

//Set up skin (used by givenewskin() function since v3, part of it before v3)
local function setupskin(p,skin)
	p.cr_skin = skin
	p.cr_time = crtime.value * TICRATE
	if cr_timeoffset.value != 0 then 
		p.cr_offset = P_RandomRange(-cr_timeoffset.value*TICRATE,cr_timeoffset.value*TICRATE)
		if p.cr_time + p.cr_offset > 3600*TICRATE then p.cr_offset = 3600*TICRATE - p.cr_time
		elseif p.cr_time + p.cr_offset < TICRATE then p.cr_offset = TICRATE - p.cr_time end
		p.cr_time = $1 + p.cr_offset
	else
		p.cr_offset = 0
	end
	R_SetPlayerSkin(p,p.cr_skin)
	if p.mo.state >= S_PLAY_SUPER_TRANS1 and p.mo.state <= S_PLAY_SUPER_TRANS6 then p.mo.state = S_SPAWNSTATE end
end

//Give a new skin to a player
local function givenewskin(p)
	if #skinlist_tab < 2 then return end
	local isskinok=false
	local isfloat=false
	local rnum=0
	if p.secondjump == 1 and p.charability == CA_FLOAT then isfloat = true end
	//Get new current skin
	if not skintabindex(p.cr_next)
		while not isskinok do
			rnum=P_RandomRange(1,#skinlist_tab)
			if R_SkinUsable(p,skinlist_tab[rnum]) and skinlist_tab[rnum] != p.cr_skin then
				setupskin(p,skinlist_tab[rnum])
				isskinok=true
			end
		end
		if p.cr_next and cr_nextskin.value == 1 then CONS_Printf(p,"Skin '"..p.cr_next.."' not found, selecting '"..p.cr_skin.."' instead") end
	else
		setupskin(p,p.cr_next)
	end
	//Get new next skin
	local isnextok=false
	while not isnextok do
		rnum=P_RandomRange(1,#skinlist_tab)
		if R_SkinUsable(p,skinlist_tab[rnum]) and skinlist_tab[rnum] != p.cr_skin then
			p.cr_next = skinlist_tab[rnum]
			isnextok=true
		end
	end
	if isfloat then 
		if p.charability != CA_FLOAT then p.secondjump = 0 end
	end
end

//cr_enable command code
COM_AddCommand("cr_enable", function(p,ena)
	if ena == "yes" or ena == "1" then
		crandom_enable=true
	elseif ena == "no" or ena == "0" then
		crandom_enable=false
	else
		if crandom_enable then
			CONS_Printf(p,"cr_enable - Enable/Disable Character Randomizer\nCurrently enabled")
		else
			CONS_Printf(p,"cr_enable - Enable/Disable Character Randomizer\nCurrently disabled")
		end
	end
end, 1)

//cr_skin command code
COM_AddCommand("cr_skin", function(p,cmd,arg)
	if cmd == "list" then
		if skinlist_str == "" then CONS_Printf(p,"Skin list is empty!") else CONS_Printf(p,"Skin list: "..skinlist_str) end
	elseif cmd == "add" then
		if isskinvalid(arg) then
			if skintabindex(arg) then
				CONS_Printf(p,"Skin '"..arg.."' is already on the skin list")
			else
				table.insert(skinlist_tab,str)
				makeskinstr()
				CONS_Printf(p,"Skin '"..arg.."' added")
			end
		else
			CONS_Printf(p,"Skin '"..arg.."' is invalid")
		end
	elseif cmd == "remove" then
		if isskinvalid(arg) then
			if skintabindex(arg) then
				table.remove(skinlist_tab,skintabindex(arg))
				makeskinstr()
				CONS_Printf(p,"Skin '"..arg.."' has been removed from the list")
			else
				CONS_Printf(p,"Skin '"..arg.."' is not on the skin list")
			end
		else
			CONS_Printf(p,"Skin '"..arg.."' is invalid")
		end
	elseif cmd == "reload" then
		getskinlist(p)
		CONS_Printf(p,"Skin list reloaded!")
	else
		CONS_Printf(p,"cr_skin commands: ")
		CONS_Printf(p,"cr_skin list - print skin list")
		CONS_Printf(p,"cr_skin add <skin name> - add skin to skin list")
		CONS_Printf(p,"cr_skin remove <skin name> - remove skin from skin list")
		CONS_Printf(p,"cr_skin reload - reload the skin list (add all available skins to the list)")
	end
end, 1)

//Checks if CR is enabled and if player has an object and counts down to 0
//If player changes their skin manually, it will be reverted to the randomly selected one
addHook("PlayerThink", function(p)
	if not crandom_enable or modeattacking then return end
	if not p.mo then return end
	if not P_IsValidSprite2(p.mo, p.mo.sprite2) then p.mo.sprite2 = SPR2_STND end
	if cr_sa2random.value == 1 then //If SA2 Rando mode is enabled
		if not p.cr_skin then givenewskin(p) end
		if p.mo.skin != p.cr_skin then R_SetPlayerSkin(p,p.cr_skin) end
		return 
	end
	if not p.cr_offset then p.cr_offset = 0 end
	if p.cr_gamemap != gamemap then p.cr_gamemap = gamemap end
	if p.exiting > 0 then p.cr_time = 0; return end
	if p.mo.health < 1 then p.cr_time = 0; return end
	if mapheaderinfo[gamemap].typeoflevel & TOL_NIGHTS then return end
	if not p.cr_skin or not p.cr_time then givenewskin(p) end
	p.cr_time = p.cr_time - 1
	if p.cr_offset > 0 and cr_timeoffset.value > 0 then
		if p.cr_offset > cr_timeoffset.value then p.cr_offset = cr_timeoffset.value*TICRATE end
		if p.cr_time > (crtime.value * TICRATE) + p.cr_offset then p.cr_time = crtime.value * TICRATE + p.cr_offset end
	else
		if p.cr_time > crtime.value * TICRATE then p.cr_time = crtime.value * TICRATE end
	end
	if p.cr_time < 1 or cr_ultimate.value == 1 then givenewskin(p) end
	if p.mo.skin != p.cr_skin then R_SetPlayerSkin(p,p.cr_skin) end
end)

//If SA2 Rando mode (only change character on level change) is enabled and map changes, then change skin
addHook("MapLoad", function(mapnum)
	if modeattacking then return end
	if not crandom_enable or cr_sa2random.value == 0 then return end
	if mapheaderinfo[mapnum].typeoflevel & TOL_NIGHTS then return end
	for p in players.iterate do
		if not p.cr_gamemap then p.cr_gamemap = -1 end
		if p.cr_gamemap != mapnum then //Prevent SP deaths from changing skin
			givenewskin(p)
			p.cr_gamemap = mapnum
		end
	end
end)

//Draws the countdown clock, but checks the same stuff as "PlayerThink" hook before drawing it
hud.add(function(v,p,cam)
	if cr_sa2random.value == 1 or cr_showtimer.value == 0 then return end
	if not crandom_enable or modeattacking then return end
	if not p.mo then return end
	if p.exiting > 0 return end
	if p.mo.health < 1 then return end
	if mapheaderinfo[gamemap].typeoflevel & TOL_NIGHTS then return end
	if not p.cr_skin or not p.cr_time then return end
	if cr_ultimate.value == 0 then
		v.timemin = tostring(G_TicsToMinutes(p.cr_time,true))
		v.timesec = tostring(G_TicsToSeconds(p.cr_time))
		if v.timesec:len() < 2 then v.timesec = "0"..v.timesec end
		v.timecsec = tostring(G_TicsToCentiseconds(p.cr_time))
		if v.timecsec:len() < 2 then v.timecsec = "0"..v.timecsec end
		if p.cr_time < TICRATE*crtime.value/10 then
			v.timestr = "Skin time left: \x85"..v.timemin..":"..v.timesec.."."..v.timecsec
		elseif p.cr_time < TICRATE*crtime.value/4 then 
			v.timestr = "Skin time left: \x82"..v.timemin..":"..v.timesec.."."..v.timecsec	
		else
			v.timestr = "Skin time left: \x83"..v.timemin..":"..v.timesec.."."..v.timecsec
		end
		if p.cr_time < 1 then v.timestr = "Skin time left: \x85".."0:00.00" end
		v.drawString(300,10,v.timestr,V_SNAPTORIGHT|V_PERPLAYER,"right")
		if cr_nextskin.value == 1 and p.cr_next then v.drawString(300,20,"Next skin: \x86"..p.cr_next,V_SNAPTORIGHT|V_PERPLAYER,"right") end
	else
		v.drawString(300,10,"\x85".."IDENTITY CRISIS MODE",V_SNAPTORIGHT|V_PERPLAYER,"right")
	end
end, "game")

//Load skin list immediately after the mod has been loaded
getskinlist()